@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: acpi-linux.c,v 1.13 2005/05/10 05:30:22 ryanl Exp $
+ $Id: acpi-linux.c 10525 2007-11-16 08:32:37Z callum $
  */
 
 /*
@@ -142,7 +142,7 @@ read_string (GHashTable *hash, const char *key)
  * result in acpiinfo->ac_online. */
 static gboolean update_ac_info(struct acpi_info * acpiinfo)
 {
-  char ac_state[60];
+  gchar *ac_state = NULL;
   DIR * procdir;
   struct dirent * procdirentry;
   char buf[BUFSIZ];
@@ -160,10 +160,11 @@ static gboolean update_ac_info(struct acpi_info * acpiinfo)
     if (procdirentry->d_name[0]!='.')
      {
       have_adaptor = TRUE;
-      strcpy(ac_state,"/proc/acpi/ac_adapter/");
-      strcat(ac_state,procdirentry->d_name);
-      strcat(ac_state,"/");
-      strcat(ac_state,acpiinfo->ac_state_state);
+      ac_state = g_strconcat("/proc/acpi/ac_adapter/",
+			     procdirentry->d_name,
+			     "/",
+			     acpiinfo->ac_state_state,
+			     NULL);
       hash = read_file (ac_state, buf, sizeof (buf));
       if (hash && !acpiinfo->ac_online)
        {
@@ -172,6 +173,7 @@ static gboolean update_ac_info(struct acpi_info * acpiinfo)
         acpiinfo->ac_online = s ? (strcmp (s, "on-line") == 0) : 0;
         g_hash_table_destroy (hash);
        }
+      g_free(ac_state);
      }
    }
 
@@ -190,8 +192,7 @@ static gboolean update_ac_info(struct acpi_info * acpiinfo)
  * the total capacity, which is stored in acpiinfo. */
 static gboolean update_battery_info(struct acpi_info * acpiinfo)
 {
-  char batt_info[60];
-  gulong acpi_ver;
+  gchar* batt_info = NULL;
   GHashTable *hash;
   DIR * procdir;
   struct dirent * procdirentry;
@@ -201,23 +202,6 @@ static gboolean update_battery_info(struct acpi_info * acpiinfo)
   acpiinfo->low_capacity = 0;
   acpiinfo->critical_capacity = 0;
 
-  hash = read_file ("/proc/acpi/info", buf, sizeof (buf));
-  if (!hash)
-    return FALSE;
-
-  acpi_ver = read_ulong (hash, "version");
-  g_hash_table_destroy (hash);
-
-  if (acpi_ver < (gulong)20020208) {
-    acpiinfo->ac_state_state = "status";
-    acpiinfo->batt_state_state = "status";
-    acpiinfo->charging_state = "state";
-  } else {
-    acpiinfo->ac_state_state = "state";
-    acpiinfo->batt_state_state = "state";
-    acpiinfo->charging_state = "charging state";
-  }
-
   procdir=opendir("/proc/acpi/battery/");
   if (!procdir)
     return FALSE;
@@ -226,9 +210,10 @@ static gboolean update_battery_info(struct acpi_info * acpiinfo)
    {
     if (procdirentry->d_name[0]!='.')
      {
-      strcpy(batt_info,"/proc/acpi/battery/");
-      strcat(batt_info,procdirentry->d_name);
-      strcat(batt_info,"/info");
+      batt_info = g_strconcat("/proc/acpi/battery/",
+			      procdirentry->d_name,
+			      "/info",
+			      NULL);
       hash = read_file (batt_info, buf, sizeof (buf));
       if (hash)
        {
@@ -237,6 +222,7 @@ static gboolean update_battery_info(struct acpi_info * acpiinfo)
         acpiinfo->critical_capacity += read_long (hash, "design capacity low");
         g_hash_table_destroy (hash);
        }
+      g_free(batt_info);
      }
    }
   closedir(procdir);
@@ -252,16 +238,40 @@ static gboolean update_battery_info(struct acpi_info * acpiinfo)
  * initializes the stored battery and AC adapter information. */
 gboolean acpi_linux_init(struct acpi_info * acpiinfo)
 {
+  GHashTable *hash;
+  char buf[BUFSIZ];
+  gchar *pbuf;
+  gulong acpi_ver;
   int fd;
 
   g_assert(acpiinfo);
+
+  if (g_file_get_contents ("/sys/module/acpi/parameters/acpica_version", &pbuf, NULL, NULL)) {
+    acpi_ver = strtoul (pbuf, NULL, 10);
+    g_free (pbuf);
+  } else if (hash = read_file ("/proc/acpi/info", buf, sizeof (buf))) {
+      acpi_ver = read_ulong (hash, "version");
+      g_hash_table_destroy (hash);
+  } else
+      return FALSE;
+
+  if (acpi_ver < (gulong)20020208) {
+    acpiinfo->ac_state_state = "status";
+    acpiinfo->batt_state_state = "status";
+    acpiinfo->charging_state = "state";
+  } else {
+    acpiinfo->ac_state_state = "state";
+    acpiinfo->batt_state_state = "state";
+    acpiinfo->charging_state = "charging state";
+  }
+
+  if (!update_battery_info(acpiinfo) || !update_ac_info(acpiinfo))
+    return FALSE;
   
   fd = open("/proc/acpi/event", 0);
   if (fd >= 0) {
     acpiinfo->event_fd = fd;
     acpiinfo->channel = g_io_channel_unix_new(fd);
-    update_battery_info(acpiinfo);
-    update_ac_info(acpiinfo);
     return TRUE;
   }
 
@@ -273,8 +283,6 @@ gboolean acpi_linux_init(struct acpi_info * acpiinfo)
     if (connect(fd, (struct sockaddr *) &addr, sizeof(addr)) == 0) {
       acpiinfo->event_fd = fd;
       acpiinfo->channel = g_io_channel_unix_new(fd);
-      update_battery_info(acpiinfo);
-      update_ac_info(acpiinfo);
       return TRUE;
     }
   }
@@ -323,7 +331,6 @@ gboolean acpi_process_event(struct acpi_info * acpiinfo)
 {
     gsize i;
     int evt;
-    gboolean result = FALSE;
     GString *buffer;
     GError *gerror=NULL;
     buffer=g_string_new(NULL);
@@ -333,17 +340,18 @@ gboolean acpi_process_event(struct acpi_info * acpiinfo)
     evt = parse_acpi_event(buffer);
       switch (evt) {
         case ACPI_EVENT_AC:
-          update_ac_info(acpiinfo);
-          result = TRUE;
-          break;
+          return update_ac_info(acpiinfo);
         case ACPI_EVENT_BATTERY_INFO:
-          update_battery_info(acpiinfo);
-          result = TRUE;
-          break;
+          if (update_battery_info(acpiinfo)) {
+            /* Update AC info on battery info updates.  This works around
+             * a bug in ACPI (as per bug #163013).
+             */
+            return update_ac_info(acpiinfo);
+          }
+          /* fall-through */
+        default:
+          return FALSE;
       }
-    
-
-    return result;
 }
 
 /*
@@ -356,7 +364,7 @@ gboolean acpi_linux_read(struct apm_info *apminfo, struct acpi_info * acpiinfo)
   guint32 rate;
   gboolean charging;
   GHashTable *hash;
-  char batt_state[60];
+  gchar* batt_state = NULL;
   DIR * procdir;
   struct dirent * procdirentry;
   char buf[BUFSIZ];
@@ -387,10 +395,11 @@ gboolean acpi_linux_read(struct apm_info *apminfo, struct acpi_info * acpiinfo)
    {
     if (procdirentry->d_name[0]!='.')
      {
-      strcpy(batt_state,"/proc/acpi/battery/");
-      strcat(batt_state,procdirentry->d_name);
-      strcat(batt_state,"/");
-      strcat(batt_state,acpiinfo->batt_state_state);
+      batt_state = g_strconcat("/proc/acpi/battery/",
+			       procdirentry->d_name,
+			       "/",
+			       acpiinfo->batt_state_state,
+			       NULL);
       hash = read_file (batt_state, buf, sizeof (buf));
       if (hash)
        {
@@ -404,6 +413,7 @@ gboolean acpi_linux_read(struct apm_info *apminfo, struct acpi_info * acpiinfo)
 	rate += read_long (hash, "present rate");
         g_hash_table_destroy (hash);
        }
+      g_free(batt_state);
      }
    }
   closedir(procdir);
