@@ -23,7 +23,7 @@
 #define __GVA_PREFERENCES_H__
 
 #include <glib.h>
-#include <gtk/gtkdialog.h>
+#include <gtk/gtk.h>
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 #ifdef HAVE_GST10
@@ -44,7 +44,7 @@ G_BEGIN_DECLS
 			    GnomeVolumeAppletPreferencesClass))
 #define GNOME_VOLUME_APPLET_IS_PREFERENCES(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOME_VOLUME_APPLET_TYPE_PREFERENCES))
-#define GNOME_VOLUME_APPLET_IS_PREFERENCES_CLASS(obj) \
+#define GNOME_VOLUME_APPLET_IS_PREFERENCES_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_VOLUME_APPLET_TYPE_PREFERENCES))
 
 typedef struct _GnomeVolumeAppletPreferences {
