@@ -67,6 +67,9 @@ stickynote_show_notes (gboolean visible)
 	StickyNote *note;
 	GList *l;
 
+    if (stickynotes->visible == visible) return;
+    stickynotes->visible = visible;
+
 	for (l = stickynotes->notes; l; l = l->next)
 	{
 		note = l->data;
@@ -74,6 +77,12 @@ stickynote_show_notes (gboolean visible)
 	}
 }
 
+static void
+stickynote_toggle_notes_visible ()
+{
+    stickynote_show_notes(!stickynotes->visible);
+}
+
 /* Applet Callback : Mouse button press on the applet. */
 gboolean
 applet_button_cb (GtkWidget         *widget,
@@ -87,7 +96,7 @@ applet_button_cb (GtkWidget         *widget,
 	}
 	else if (event->button == 1)
 	{
-		stickynote_show_notes (TRUE);
+		stickynote_toggle_notes_visible ();
 		return TRUE;
 	}
 
@@ -232,6 +241,8 @@ void applet_destroy_cb (PanelApplet *panel_applet, StickyNotesApplet *applet)
 {
 	GList *notes;
 	
+	stickynotes_save_now ();
+
 	if (applet->destroy_all_dialog != NULL)
 		gtk_widget_destroy (applet->destroy_all_dialog);
 
